      program pgswmfa
*
*     multiple filter analysis of a surface wave signal
*     author: Charles J. Ammon, Saint Louis University
*
*     version: 3.9.0 - February, 1999
*
*     Version history:
*
*     version 2.2.2 - added batch command
*     version 2.2.3 - fixed leapyear calculation bug
*     version 2.2.4 - changed the dimensions (750 velocities,
*                     128 maximum periods)
*     version 2.2.5 - changed the SAC output to use instantaneous
*                     period
*     version 2.2.6 - fixed a bug in the 'v' command (spline and peak
*                     velocities were switched) - minor problem.
*
*     version 2.2.7 - changed 'A', 'D', 'X' commands to use cursor
*                     keys for low-high-refresh.
*
*     version 2.3.0 - changed the velocity pick to use envelope and
*                     a quadratic fit to peak of envelope (interpolates).
*                     doesn't make a big difference for finely sampled
*                     velocity functions
*
*     version 2.3.1 - changed default image scaling to use m^0.5 and added
*                     'z' command to allow use to change the value.
*                     now using instantaneous at the peak, not spline
*                     also, the peak velocities are measured off the 
*                     a quadratic fit to the peak of the envelope to allow
*                     smooth interpolation between time samples. changed
*                     xlabel on the image to 'Filter Center Period (s)'
*    version 3.0.0    Fixed a scaling bug in the seismogram output
*                     A major problem for output seismograms, and amplitudes
*                     but dows not affect the group velocities.
*
*    version 3.5.0    Added George Randall's "batch run" option that runs
*                     the code without user interaction. For each seismogram
*                     in the list you get a plot and a dispersion file.
*
*     version 3.6.0   minor changes and checks suggested by velasco.
*
*     version 3.6.1   cosmetic change to adjust scaling in mode isolation plots.
*
*     version 3.7.0   Minor change to search for envelope peak only in the specified
*                     range of group velocities. This is the way I started, but 
*                     when I started fitting the envelope with a quadratic, i expanded
*                     the search over the whole signal - this can be annoying and
*                     cause uneeded editing. Affects loop 80.
*
*     version 3.8.0   switched to average method of computing the peak ifreq. was
*                     using the expected value, but that was experimental and had
*                     a bug.
*
*                     also added the command line argument '-f' for fixed gaussian width
*                          selection.
*
*                     also added command 'e' for dumping the envelope, ifreq, and 
*                          filtered seis into sac files called ex_seis, ex_envl, ex_iper.
*
*     version 3.8.2   fixed a sign error  group-velocity windowing 
*                         introduced in version 3.7.0
*
*     version 3.8.3   changed put_seis.f format ( * -> '(a)' ) 02/08/99
*
*
*     version 3.9.0   added the ability (-n) to use pre-sginal noise in 
*                     spectral plot (04/2001) and added the ability to match filter
*                     with the reference curve. Also fixed minor annoyance when
*                     in prompt for postscript file name.
*

      include 'swanalysis.h'
      
       real work(maxtpts)
       INTEGER PGOPEN,variable_gwidth
       logical batch            
       character*1 atype, yn
       character*6 outfile 
       character*32 slabel, afmt
       character*256 theFiles(500), graphics_file
             
       variable_gwidth=1
       estimate_noise = 0

       nargc = iargc()
       if ( nargc .gt. 0 ) then
	  call chkopts(nargc,batch,variable_gwidth,estimate_noise)
	  write(stdout,*)'estimate noise = ', estimate_noise
       endif
       
       stdin = 5
       stdout = 6
       stderr = 6
                     
       iX = 0
       iPS = 1
       iCPS = 2
       iTEK = 3
       if ( batch ) then
          idevice = iCPS
	else
	  idevice = iX
       endif
       ideviceChanged = 1

       bright = 0.5
       contrast = 0.5
       
       a_image_exponent = 0.5
       
       
       iblack = 0
       iwhite = 1
       ired = 2
       igreen = 3
       iblue = 4
       icyan = 5
       imagenta = 6
       iyellow = 7
       iorange = 8
       igreyel = 9
       igrecya = 10
       iblucya = 11
       iblumag = 12
       iredmag = 13
       idarkgray = 14
       ilightgray = 15
       
       isolid = 1
       
       inunit = 10
       ounit = 11
       name_unit = 12
       
       forward = 1
       inverse = -1
       
       isolated = 0
       
       ishow_sqrt = 0
       
       i_examine_period = 0
*
*********************************************************************************
*      read in the file names
*********************************************************************************
*       
       i = 1
       INQUIRE(file='list', EXIST = lexists)
       if(lexists) then
          open(unit=name_unit,file='list')
2         continue
          read(name_unit,'(a72)',end=5) theFiles(i)
          i = i + 1
          go to 2
       else
          write(stdout,*)'No file named "list" exists.'
          write(stdout,*)'Enter the name of the seismogram file'
	  read(stdin,*) theFiles(1)
	  i = i +1
       end if
*
5      continue
       close(name_unit)
       nfiles = i - 1
       ncount = 1
*       
       call listmenu
*
*********************************************************************************
*
*      This is where the file loop begins
*
10     continue
*
       if(ncount .gt. nfiles) go to 9999
       seisfile = '                                              '
       seisfile = theFiles(ncount)
*
*********************************************************************************
*      build a graphics file name = ps.seisfile
*********************************************************************************
*
       islen = lnblnk(seisfile)
       graphics_file(1:islen+25) = 
     &          '?Plot file name (e.g. ps.'//seisfile(1:islen)
       islen = islen + 25
*       
       if ( batch ) then
          islen = lnblnk(seisfile)
          graphics_file = 'ps.'//seisfile(1:islen)//' '
          islen = lnblnk(graphics_file)
	endif
*       
       ncount = ncount + 1
*
*********************************************************************************
*     Input the Seismogram
*
      do 20 i = 1, maxtpts
	seis(i) = 0.0
20    continue
*
      call fgetseis()
*
********************************************************************************* 
*     Normalize the seismogram
*
      call normalize(seis,ntpts,normalizer,nerror)
      if(nerror .ne. 0) then
	 write(stdout,*)'PROBLEM WITH SEISMOGRAM - SKIPPING'
	 go to 10
      end if
*
*********************************************************************************
*     Compute parameters for Fourier Transform
*
*********************************************************************************
      nfpts = 1
50    nfpts = nfpts * 2
      if(nfpts .lt. ntpts) go to 50
      
      if(nfpts .gt. maxtpts)then
	write(stdout,*)'Too many points in ',
     &      seisfile(1:ll)
        go to 10
      end if
      halfpts = nfpts / 2
      fnyquist = 1 / (2*dt)
      nyqpt = halfpts
      df = 1.0 / (nfpts * dt)
*
*********************************************************************************
*     Get the details for windowing and velocity and period ranges
*********************************************************************************
*
      is_overtone = 0
      ll = lnblnk(seisfile)
      write(afmt,'(a6,i3.3,a1)') '(a17,a',ll,')'
      write(stdout,*)' '
      write(stdout,afmt)'Processing file: ', seisfile
      write(stdout,*)' '
*
51    continue
      if(seisfile(ll:ll) .eq. 'z'.or. seisfile(ll:ll) .eq. 'Z'
     & .or. 
     &   seisfile(ll:ll) .eq. 'r'.or. seisfile(ll:ll) .eq. 'R'
     &   ) then
        itype = 1
      else if(seisfile(ll:ll) .eq. 't'
     &   .or. seisfile(ll:ll) .eq. 'T') then
        itype = 2
      else
        write(stdout,*) 'Rayleigh (1) or Love (2)?'
        read(stdin,'(a1)') atype
        if(atype .eq. '1' .or. atype .eq. 'r') itype = 1
        if(atype .eq. '2' .or. atype .eq. 'l') itype = 2
      end if
c
      if((itype .ne. 1) .and. (itype .ne. 2)) go to 51
*
*********************************************************************************
       call getenv( 'PREM_R_GVEL' , prem_rayl_file )
       call getenv( 'PREM_L_GVEL' , prem_love_file )
c
*********************************************************************************
c     read in the prem reference values for rayleigh or love
*********************************************************************************
c      
      nref = 44
      ihasrefcurve = 0
      if(itype .eq. 1)then
         INQUIRE(file=prem_rayl_file, EXIST = lexists)
	 if(lexists)then
	    open(unit=inunit,file=prem_rayl_file)
	    do 54 i = 1, nref
	      read(inunit,*) prem_t(i), prem_g(i)
	      prem_t(i) = log10(prem_t(i))
	      prem_g(i) = prem_g(i)
54	    continue
           close(inunit)
	   ihasrefcurve = 1
         end if   
      else if(itype .eq. 2)then
         INQUIRE(file=prem_love_file, EXIST = lexists)
	 if(lexists)then
	    open(unit=inunit,file=prem_love_file)
	    do 55 i = 1, nref
	      read(inunit,*) prem_t(i), prem_g(i)
	      prem_t(i) = log10(prem_t(i))
	      prem_g(i) = prem_g(i)
55	    continue
            close(inunit)
	   ihasrefcurve = 1
         end if      
      
      end if
*
*********************************************************************************
*     These can be read (and will be) from a file named 'in.swmfa'
*      if it exists
*********************************************************************************
*
      INQUIRE(file='in.swmfa', EXIST = lexists)
      if(lexists)then
          write(stdout,'(a,/)')'Read parameters from in.swmfa.'
          open(unit=inunit,file='in.swmfa')
          read(inunit,*) gwidth
          read(inunit,*) vgmin,vgmax
          read(inunit,*) pmin,pmax
          read(inunit,*) nperiods
          close(inunit)
      else
         write(stdout,*) 'What is the gaussian width factor?'
         read(stdin,*) gwidth
           
         write(stdout,*) 'What is the group velocity range (min,max)?'
         read(stdin,*) vgmin,vgmax
      
         write(stdout,*) 'Enter the min and max periods?'
         read(stdin,*) pmin,pmax
      
         write(stdout,*) 'Enter the number of periods (max = 256)?'
         read(stdin,*) nperiods
      end if
*      
      tmin = distance / vgmax
      tmax = distance / vgmin
      dvg = (vgmax - vgmin) / (maxvel - 1)
      nvel = maxvel
*
      pmax = log10(pmax)
      pmin = log10(pmin)
      dlogp = (pmax - pmin) / (nperiods - 1)
      do 56 i = 1,nperiods
        use_it(i) = 1
56    continue

*
      if(estimate_noise .eq. 1)then
         call est_noise(xnoise,ynoise,seis,ntpts,dt,beg,a_pick,nnoise,iok)
	 do 57 i =1,nnoise
	    ynoise(i) = ynoise(i) + log10(normalizer)
57       continue
      end if
*
*     get the USGS low and high noise polygon for spectral plot
*
      if((estimate_noise .ne. 1) .or. (iok .eq. -1))then
        call get_usgs_noise(pmin,pmax,xnoise,ynoise,50,110,100.0)
	nnoise = 101
      end if
*      
c     check to make sure we have enough points in the trace.
c     if we don't "zero" pad by changing the number of points
c      
      if(ntpts .lt. (tmax-or_beg) / dt) ntpts = (tmax-or_beg) / dt + 1
      if(tmax .gt. maxtpts)then
        write(stdout,*)'too many points in the trace:',seisfile
        go to 10
      end if
*
*     store the original seismogram in the array orig_seis (so
*          that we have an "undo" potential).
*      
      do 58 i = 1, maxtpts
        orig_seis(i) = seis(i)
58    continue      
*
250   continue
*********************************************************************************
*     Transform the original time series
*********************************************************************************
      call realft(seis,halfpts,forward)
      scalefactor = dt
      do 60 i = 1, nfpts
        seis(i) = seis(i) * scalefactor
	thespec(i) = seis(i)
60    continue
*
*********************************************************************************
*     Compute image for contouring
*********************************************************************************
      magnitude = 0.0
      cmax = -1e20
      cmin =  1e20
      do 100 i = 1, nperiods
      
         do 70 j = 1, maxtpts
            wind_seis(j) = 0.0           
70       continue

         period = 10**(real(i-1)*dlogp + pmin)
         periods(i) = log10(period)

         fcenter = 1 / period

	 if(variable_gwidth .eq. 1)then
	    call getalpha(distance,period,a0)
c 	    write(stdout,*)period, a0
	 else
	    a0 = gwidth
	 end if
	 
	 call gfilter(seis,wind_seis,a0,fcenter,df,halfpts)
         
         call realft(wind_seis,halfpts,inverse)
         scalefactor = 2 * df * normalizer
         do 71 k = 1, nfpts
            wind_seis(k) = wind_seis(k) * scalefactor
71       continue
*
*********************************************************************************
*         uncomment these to see a filtered seismogram
*
*         write(outfile,'(a4,i2.2)')'filt',i
*         call wsac1(outfile,wind_seis,ntpts,or_beg,dt,nerr)
*********************************************************************************
*         
	  call envelope_plus(wind_seis,ntpts,dt,envelope,the_ifrq)
*         
*********************************************************************************
*         uncomment these to see a seismogram envelope
*
*         write(outfile,'(a4,i2.2)')'envl',i
*         call wsac1(outfile,envelope,ntpts,or_beg,dt,nerr)
*         write(outfile,'(a4,i2.2)')'ifrq',i
*         call wsac1(outfile,the_ifrq,ntpts,or_beg,dt,nerr)
*********************************************************************************
*         
         theGroupVels(i) = 0
         rmax = -1e20
         do 75 k = 1, nvel 
*
*           convert from velocity to time and read the values
*	    
	    vg = vgmin + real(k-1)*dvg
	    tg = distance / vg
	    k0 = int( (tg - or_beg) / dt  + 0.49 )
	    if(k0 .le. ntpts .and. k0 .gt. 0) then
	        magnitude = envelope(k0)
		frequency = the_ifrq(k0)
	    else
	        magnitude = 0
	    end if
*
*           store the images in the arrays
*	    
	    fbuffer(i,k) = magnitude**a_image_exponent
*    
	    instfrq(i,k) = frequency
	    if(magnitude .gt. cmax) cmax = magnitude
	    if(magnitude .lt. cmin) cmin = magnitude
*	    
*           store info for the max amplitude for this period
*
	    if(magnitude .gt. rmax)then
	      rmax = magnitude
	      theGroupVels(i) = vg
	      theAmplitudes(i) = log10(magnitude)
	      theInstPeriod(i) = 1.0 / frequency
	      imax = k0
	      kmax = k
	    end if

75       continue
*
*        go back to the envelope to find the right velocity and period
*
*        limit the search by the user's gvel windows
*
         amax = -1e9
	 ib80 = (distance / vgmax - or_beg) / dt + 1
	 ie80 = (distance / vgmin - or_beg) / dt + 1
	 if(ib80 .lt. 1) ib80 =1
	 if(ie80 .gt. ntpts) ie80 = ntpts
         do 80 k = ib80, ie80
	    if(envelope(k) .gt. amax) then
	      amax = envelope(k)
	      frequency = the_ifrq(k)
	      theInstPeriod(i) = 1.0 / frequency
	      kmax = k
	    end if
80       continue
*
*        Fit a quadratic to the amplitudes at these velocities.
*         Effectively this interpolates the peak of the envelope
*         across three points
*
         if(kmax .gt. 1 .and. kmax .lt. ntpts)then
           qa(1) = envelope(kmax-1)/envelope(kmax)
           qa(2) = 1.0
           qa(3) = envelope(kmax+1)/envelope(kmax)
           tref  = real(kmax)*dt + or_beg 
	   avg_freq =  ( qa(1)*the_ifrq(kmax-1) +
     &                   qa(2)*the_ifrq(kmax)   +
     &	                 qa(3)*the_ifrq(kmax+1) ) / 
     &                   (qa(1) + qa(2) + qa(3) )
	   qv(1) = -dt
           qv(2) = 0 
           qv(3) = dt
*	   
	   call quad_peak(qv,qa,3,vquad)
	   theGroupVels(i) = distance / (tref + vquad)
	 end if
*
*	 compute the average instantaneous frequency 
*
*        Note that this is not used right now.
	 call wt_avg(the_ifrq,envelope,ntpts,dt,frequency)
*
*        The above formula has proven to be unstable at times,
*            using the simple average is equivalent with good data,
*            and seems more stable with other observations.
*
 	 theInstPeriod(i) = 1.0 / avg_freq
*
*        Check the shape of the envelope. Only the centroid is
*          used right now.
*
         call skew_kurt(envelope,ntpts,centroid,skew,curt)
	 
	 tg = centroid * dt + or_beg
	 centroidGroupVels(i) = distance / tg
*
*        Compute the period with the maximum spectral amplitude.
*          Not used.
*
         call ampsp_centroid(wind_seis,nfpts,fcentroid,fpeak,dt)
	 peakPeriods(i) = 1 / fpeak
*
*        find the velocities at 95% of the peak value 
*
         errfraction = 0.95
         call bound_it(envelope,ntpts,imax,ih,il,errfraction)
*
	 tg = distance/(theGroupVels(i))-real(imax-ih)*dt
	 gvhigh(i) = distance / tg
*
	 tg = distance/(theGroupVels(i))+real(il-imax)*dt
	 gvlow(i)  = distance / tg
	 
	 if(i_examine_period .eq. 1)then
	   thedif = abs(examine_period - 1/fcenter)
	   if(thedif .lt. 0.01)then
	     call export_signals(wind_seis,envelope,the_ifrq,
     &                           ntpts, delta, normalizer, work,
     &                           period)
	   i_examine_period = 0
	   endif
	 endif
* 
100    continue
*      end of loop over velocity windows
*
*********************************************************************************
*     Transform back the original time series
*
*********************************************************************************
      call realft(seis,halfpts,inverse)
      scalefactor = 2 * df
      do 200 i = 1, nfpts
        seis(i) = seis(i) * scalefactor
200    continue
*
*********************************************************************************
*
*     Graphics
*
*********************************************************************************
*     setup: compute a rotation matrix needed for the image commad
*********************************************************************************
*      
      tr(1) = 0
      tr(2) = 1
      tr(3) = 0
      tr(4) = 0
      tr(5) = 0
      tr(6) = 1
*
      ncontours = 12
      if(idevice .eq. iTEK) ncontours = 8
*
*********************************************************************************
*
300    continue
*
*      itermstat is the terminal device identifier
*      ipsstat is the postscript file identifier
*
       if(ideviceChanged .eq. 1) then
*
         if(idevice .eq. iX) then
            itermstat = pgopen('/xwindow')
	    if(itermstat .le. 0)then
	      write(stdout,*)'Problem opening X device.'
302           itermstat = pgopen('? ')
	      if(itermstat .le. 0) go to 302
            end if
         else if(idevice .eq. iPS) then
            graphics_file(1:islen+3)=graphics_file(1:islen)//'/ps'
	    ipsstat = pgopen(graphics_file)
	    if(ipsstat .le. 0)then
	      write(stdout,*)'Problem opening PS device.'
	      stop
            end if
	    call pgslct(ipsstat)
         else if(idevice .eq. iCPS) then
            graphics_file(1:islen+6)=graphics_file(1:islen)//'/cps )'
	    ipsstat = pgopen(graphics_file)
	    if(ipsstat .le. 0)then
	      write(stdout,*)'Problem opening CPS device.'
	      stop
            end if
	    call pgslct(ipsstat)
         else if(idevice .eq. iTEK)then
	    itermstat = pgopen('/tk4100')
	    if(itermstat .le. 0)then
	      write(stdout,*)'Problem opening Tek device.'
	      stop
            end if
         end if
*
         ideviceChanged = 0
*
*        set up the image color table
*
          lplot_image = .true.
          call pgqcir(ic1,ic2)
          ncolors = MAX(0,ic2-ic1+1)
          if(ncolors .lt. 8) then
            write(stdout,*)'Not enough colors on device'
	    lplot_image = .false.
          end if
       end if
*      end of device change
*
*     erase the screen
*
       call pgeras
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*      The title
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
       xvsmax = 1.0 - 0.025
       xvsmin = 0.0 + 0.025
       yvsmax = 1.0 - 0.025
       yvsmin = 0.0 + 0.025
       call pgsvp(xvsmin,xvsmax,yvsmin,yvsmax)
       call pgswin(0.0,1.0,0.0,1.0)
       
       x = xvsmin
       y = yvsmax
*
*      build the station, etc. string
*
       call getkhv('kstnm',kstnm,nerr)
       call getkhv('kcmpnm',kcmpnm,nerr)
       call getnhv('nzyear',nzyear,nerr)
       call getnhv('nzjday',nzjday,nerr)
       call getnhv('nzhour',nzhour,nerr)
       call getnhv('nzmin',nzmin,nerr)
*      
       call getdate(nzyear,nzjday,imonth,iday)
*
*       write(title,304) 
*     &    'Station: ',kstnm,' Component: ',kcmpnm,
*     &    ' Date: ',nzyear,nzjday,nzhour,':',nzmin
*304    format(a9,a8,a12,a8,a7,i4,1x,i3.3,1x,i2.2,a1,i2.2)

       write(title,304)kstnm,kcmpnm,nzyear,
     &                 imonth,iday,nzjday,nzhour,nzmin
304    format('Station: ',a8,' Component: ',a8,
     &    ' Date: ',i4,1x,i2.2,'/',i2.2,' (',i3.3,')',
     &    1x,i2.2,':',i2.2)
*
*      draw the string
*
       fsize = 1.20
       cht   = 0.04
       call pgsch(fsize)
       call pgtext(x,y,title)
*
*      build the distance/azimuth label
*
       call getfhv('AZ',az,nerr)
*      
       if(variable_gwidth .eq. 1)then
          write(title,305) distance,az
       else
          write(title,306)gwidth,distance,az
       end if
       
305    format('Alpha=Variable',4x,'Distance: ',f9.1,3x,'Az: ',f6.1)     
306    format('Alpha=',f6.2,4x,'Distance: ',f9.1,3x,'Az: ',f6.1)
*
*      draw the string
*
       y = y - cht
       call pgtext(x,y,title)
       fsize = 1.0
       call pgsch(fsize)
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*      plot the spectral amplitudes
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
*     Compute the range of spectral amplitudes
*
*     skip the DC value
*      
      do 310 i = 2, halfpts
        itmp = i - 1
        xpoly(itmp) = log10(1.0 / (itmp*df + 1e-9))
	ypoly(itmp) = log10(sqrt(thespec(2*i)*thespec(2*i)+
     &                thespec(2*i-1)*thespec(2*i-1) ) )
     &             +log10(normalizer)
310   continue
      xpoly(halfpts) = log10(1.0 / (halfpts*df + 1e-9))
      ypoly(halfpts) = log10(abs(thespec(2)))+log10(normalizer)
*
       call getminmeanmax(ypoly,halfpts,smin,smean,smax)
      
       do 315 i = 1, nperiods
         itmp = (1.0 / 10**periods(i)) / df + 1
         atmp(i) = ypoly(itmp)
	 c_amps(i) = cmplx(thespec(2*itmp-1),thespec(2*itmp))
315    continue 

       offset = 0.1
       xvsmin = 0
       xvsmax = 1
       yvsmax = 0.9
       yvsmin = 0.1
*
       xvpmax = xvsmax - 0.75 * offset
       xvpmin = 0.725 * xvsmax + 0.025 * offset
       yvpmin = yvsmin + offset
       yvpmax = yvsmax - offset
*
       call pgsvp(xvpmin,xvpmax,yvpmin,yvpmax)
*
       wmin = pmin
       wmax = pmax
*
*       put a limit on the number of decades for the plot
*       
       if(smax - smin .gt. 4.0) smin = smax - 4.0
       vmin = smin
       if(vmin .lt. 0)then
           vmin = 1.05 * vmin
       else if(vmin .gt. 0)then
           vmin = 0.95 * vmin
       else
           vmin = -0.05
       end if
       vmax = smax + 0.05
*
       call pgswin(wmin,wmax,vmin,vmax)
       if(idevice .eq. iX) then
           call fillrect(idarkgray,wmin,wmax,vmin,vmax)
       end if       
*
       call pgsave
*
*      draw the usgs noise bounds
*
       call pgsave
       call pgsch(1.25)
       if(idevice .eq. iX) then
          call pgscr(ilightgray,0.7,0.7,0.7)
       else
          call pgscr(ilightgray,0.3,0.3,0.3)
       end if       
       call pgsci(ilightgray)
       call pgsfs(isolid)
       call pgline(nnoise,xnoise,ynoise)
       call pgunsa
*
*      draw the observed spectra
*       
       call pgline(halfpts,xpoly,ypoly)
*
*      draw the points
*
       call pgsave
       isnumber = -4
       isymsize = 1.0
       call pgslw(isymsize)
       call pgsch(1.0)
       call pgsfs(isolid)
       call pgsci(ired)
        do 320 i = 1, nperiods
          if(use_it(i) .eq. 1)then
            call pgpt1(periods(i),atmp(i),isnumber)
          end if
320    continue 
       call pgunsa
*   
       call pgbox('BCTSNL',0,0,'ABCTSMVL',0,0)
       call pgmtext('B',2.5,0.5,0.5,'Period (s)')
       call pgmtext('T',0.5,0.5,0.5,'Spectral Amplitude')
       call pgunsa
*      
       
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*      plot the seismogram with a velocity scale
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
*       filter the seis to the period range picked by the user
*
        do 325 i = 1,ntpts
           wind_seis(i) = seis(i)
325     continue
        do 326 i = ntpts, nfpts
           wind_seis(i) = 0
326     continue
        call realft(wind_seis,halfpts,forward)
        
        do 327 i = 1, nperiods
          if(use_it(i) .eq. 1)then
            fl = 1 / (10**periods(i))
           end if
327     continue
         do 328 i = nperiods,1,-1
          if(use_it(i) .eq. 1)then
            fh = 1 / 10**periods(i)
           end if
328     continue

       write(slabel,'(f5.2,a4,f7.2,a2)')1/fh,' to ',1/fl,' s'
*
*      apply the box-car filter
       call bcfilter(wind_seis,fl,fh,df,halfpts)
       call realft(wind_seis,halfpts,inverse)
       do 329 i = 1,nfpts
           wind_seis(i) = 2*dt*df*wind_seis(i)
329    continue
*
*      compute a scale factor for plotting
*
       call getminmeanmax(wind_seis,nfpts,xmin,xmean,xmax)
       xmax = abs(xmax)
       if(abs(xmin) .gt. xmax) xmax = abs(xmin)
*
       offset = 0.1
       xvpmax = 0.75 * xvsmax - 0.5 * offset
       xvpmin = 0.75 * xvsmax - 2.25 * offset
       yvpmin = yvsmin + offset
       yvpmax = yvsmax - offset
       
       call pgsvp(xvpmin,xvpmax,yvpmin,yvpmax)
       
       wmin = -1.05005
       wmax =  1.05005
       amin = -1.05
       amax =  1.05
       vmin = 0.9995 * vgmin
       vmax = 1.0005 * vgmax
       
       call pgswin(wmin,wmax,vgmin,vgmax)
       if(idevice .eq. iX) then
           call fillrect(idarkgray,wmin,wmax,vgmin,vgmax)
       end if       
       call pgbox('ABCTS',0,0,'ABCTSV',0,0)

       tmin = distance / vgmax - or_beg
       itmin = tmin / dt
       tmax = distance / vgmin - or_beg
       itmax = tmax / dt
       if(itmin .lt. 1) itmin = 1
       if(itmax .gt. ntpts) itmax = ntpts
       x = wind_seis(itmin)/xmax
       itmin = itmin + 1
*
       call pgsave
       ilinewidth = 1
       call pgslw(ilinewidth)
*
       do 400 i = itmin, itmax
         xpoly(i) = wind_seis(i) / xmax
         y = (i - 1) * dt + or_beg
         ypoly(i) = distance / y
400    continue
       itmp = itmax - itmin
       call pgline(itmp,xpoly(itmin),ypoly(itmin))
       call pgunsa
       call pgmtext('T',0.5,0.5,0.5,slabel)
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *       
*      plot the contours of the image
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *      
*
       call get_subimage_range(fbuffer,maxperiods,maxvel,
     &                         nvel,nperiods,use_it,cmin,cmax)
       dc = ( cmax - cmin ) / (ncontours - 1)
       do 405 i = 1, ncontours
         cntrs(i) = cmin + (i-1)*dc
405    continue	 
       
       offset = 0.1

       xvpmin = xvsmin + offset
       xvpmin = 0.075
       xvpmax = 0.75 * xvsmax - 2.5 * offset
       yvpmin = yvsmin + offset
       yvpmax = yvsmax - offset
       
       call pgsvp(xvpmin,xvpmax,yvpmin,yvpmax)
       
       xwmin = 0.9995 * pmin
       xwmax = 1.0005 * pmax
       ywmin = 0.9995 * vgmin
       ywmax = 1.0005 * vgmax
*
       call pgswin(1.0,real(nperiods),1.0,real(nvel))
*  
*      plot a color image of the period-velocity spectrum
*    
       if(lplot_image)then
*         
         call pgsave
	 call palett(2,contrast,bright)
*
         call pgimag(fbuffer,maxperiods,maxvel,1,nperiods,1,nvel,cmin,cmax,tr)
*
         call pgwedg('BI',4.0,3.0,cmin,cmax,'Amplitude')
	 call pgunsa
       end if
*
*      plot the contours
*
       call pgsave
       if(idevice .eq. iX) call pgsci(iblack)
       call pgcont(fbuffer,maxperiods,maxvel,1,nperiods,1,nvel,cntrs,ncontours,tr)
       call pgunsa
*
       call pgsave
       call pgswin(xwmin,xwmax,ywmin,ywmax)
       call pgbox('ABCTSNL',0,0,'ABCTSNV',0,0)
       call pgmtext('B',2.5,0.5,0.5,'Filter Center Period (s)')
       call pgmtext('L',2.5,0.5,0.5,'Group Velocity (km/s)')
       call pgmtext('T',0.5,0.5,0.5,'Velocity-Period Spectrum')
       call pgunsa
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*      Plot the max values for each frequency
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*      Fit the values with a spline
*
       call spline_it(periods,theGroupVels,use_it,
     &       nperiods,n_splines,splGroupVels)
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*      Look up the new instantaneous frequencies
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
      do 407 i = 1, nperiods
	if(use_it(i) .eq. 1)then
	  if(splGroupVels(i) .lt. vgmin) splGroupVels(i) = vgmin
	  if(splGroupVels(i) .gt. vgmax) splGroupVels(i) = vgmax
	  k = int( (splGroupVels(i) - vgmin) / dvg + 0.5 ) + 1
	  frequency = instfrq(i,k)
	  if(frequency .eq. 0.0)then
	    write(stdout,*) ' '
	    write(stdout,*)'!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
	    write(stdout,*)'WARNING - Instantaneous Frequency Zero!'
	    itmp = lnblnk(seisfile)
	    write(stdout,*) seisfile(1:itmp) 
            write(stdout,*)'T=',10**periods(i), ' V=',splGroupVels(i)
	    write(stdout,*) 'Using peak frequency value.'
	    write(stdout,*)'!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
	    write(stdout,*) ' '
	  else
c           not using instantaneous at the spline, but the peak
c	    theInstPeriod(i) = 1.0 / frequency
	  end if
        end if
407   continue
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
*      write(stdout,*) 'starting plot'
       call pgsave
*
       isnumber = -4
       isymsize = 2
       call pgsch(1.5)
       call pgsfs(isolid)
       call pgsci(ired)
*
       do 410 i = 1, nperiods
         if(use_it(i) .eq. 1)then
	   call pgpt1(periods(i),theGroupVels(i),isnumber)
         end if
410    continue
*
*      write(stdout,*) 'past diamonds'
       ilinewidth = 5
       call pgslw(ilinewidth)
       call pgsci(iblue)
       ifirst = 0
       do 411 i = 1, nperiods
         if(use_it(i) .eq. 1)then
	   if(ifirst .eq. 0)then
	    call pgmove(periods(i),splGroupVels(i))
	    ifirst = 1
           else
	    call pgdraw(periods(i),splGroupVels(i))
           end if
         end if
411    continue
       call pgunsa
*      write(stdout,*) 'past spline'
c
c      plot the reference model dispersion (prem)
c
       call pgsave
       call pgsci(iblack)
       if(idevice .eq. IPS .or. idevice .eq. iCPS) then
          call pgsci(iwhite)
       end if       
       isnumber = -3
       call pgsch(1.5)
       call pgsfs(isolid)
       do 412 i = 1, nref
         if(prem_t(i) .ge. pmin .and. prem_t(i) .le. pmax)then
	   call pgpt1(prem_t(i),prem_g(i),isnumber)
         end if
412    continue
       call pgunsa
c
c      plot the error bars
c
       call pgsave
       call pgsci(ired)
       hbar = (pmax-pmin)*0.01
       do 414 i = 1, nperiods
         if(use_it(i) .eq. 1)then
*	   
           call pgmove(periods(i),gvhigh(i))
	   call pgdraw(periods(i), gvlow(i))
*
           call pgmove(periods(i)-hbar,gvhigh(i))
	   call pgdraw(periods(i)+hbar,gvhigh(i))
*
           call pgmove(periods(i)-hbar,gvlow(i))
	   call pgdraw(periods(i)+hbar,gvlow(i))
	  end if
414    continue

       call pgunsa
*      write(stdout,*) 'past bounds'
*
*      call pltvel(theInstPeriod,theGroupVels,
*     &            theInstPeriod,splGroupVels, 
*     &            theInstPeriod,centroidGroupVels,
*     &            use_it,nperiods)
                  
*********************************************************************************
*     Interactive Section
*
*********************************************************************************
490    continue
*
*
*********************************************************************************
*      see if batch mode, then plot is color postscript, and write dipsersion
*      otherwise, normal interactive mode with all commmands
*      if we just made a hard copy - skip the cursor stuff
*      until the next pass.
*********************************************************************************

	if ( batch ) then
           call pgiden
           call pgclos
           ideviceChanged = 1

            call put_dsprsn(seisfile,splGroupVels,
     &           periods,theInstPeriod,
     &           c_amps,use_it,nperiods,itype,
     &           theGroupVels, gvlow, gvhigh,
     &           gwidth,variable_gwidth,
     &           isolated,pmin,pmax,vgmin,vgmax,errfraction,
     &           is_overtone)
           goto 8888
	else
*
*********************************************************************************
*      if we just made a hard copy - skip the cursor stuff
*      until the next pass.
*********************************************************************************
*       
       if(idevice .eq. iPS .or. idevice .eq. iCPS)then
          key = 'p'
       else
          call pgcurs(p,vg,key)
       end if
*
*      list the menu
*       
       if(key.eq. 'm' .or. key .eq. 'M') then
          call listmenu()
       end if
*
       if(key .eq. 'q' .or. key .eq. 'Q') goto 9999
       
       if(key .eq. 'd') then
          if(p .ge. pmin .and.  p .le. pmax .and.
     &       vg .ge. ywmin .and. vg .le. ywmax)then
                 index = (p - pmin)/dlogp + 1
                 plow = periods(index)
                 dplow = abs(p - plow)
                 dphigh = abs(p - plow - dlogp)
                 if(dphigh .lt. dplow) index = index + 1
                 p = periods(index)
                 use_it(index) = -1
                 write(stdout,*) index, p, vg
          end if
       end if
       
       if(key .eq. 'i' .or. key .eq. 'I')then
          if(p .ge. pmin .and.  p .lt. pmax .and.
     &       vg .ge. ywmin .and. vg .le. ywmax)then
		 index = (p - pmin)/dlogp + 1
                 plow = periods(index)
                 dplow = abs(p - plow)
                 dphigh = abs(p - plow - dlogp)
                 if(dphigh .lt. dplow) index = index + 1
                 p = periods(index)
                 theGroupVels(index) = vg
                 use_it(index) = 1
                 k0 = int((vg-vgmin)/dvg) + 1
                 theAmplitudes(index) = log10(fbuffer(index,k0))
                 write(stdout,*) 'Interpolated: ',
     &                index,p,vg,theAmplitudes(index)
          end if
       end if
*
*      truncate all values with period lower than the cursor value
*       
       if(key .eq. 'l' .or. key .eq. 'L' .or. key .eq. 'A')then
          if(p .ge. pmin .and.  p .le. pmax .and.
     &       vg .ge. ywmin .and. vg .le. ywmax)then
            index = (p - pmin)/dlogp + 1
            plow = periods(index)
            dplow = abs(p - plow)
            dphigh = abs(p - plow - dlogp)
            if(dphigh .lt. dplow) index = index + 1
            do 420 kk = 1, index
               use_it(kk) = -1
420         continue
          end if
       end if
*
*      truncate all values with period higher than the cursor value
*       
       if(key .eq. 'h' .or. key .eq. 'h' .or. key .eq. 'D')then
          if(p .ge. pmin .and.  p .le. pmax .and.
     &       vg .ge. ywmin .and. vg .le. ywmax)then
            index = (p - pmin)/dlogp + 1
            plow = periods(index)
            dplow = abs(p - plow)
            dphigh = abs(p - plow - dlogp)
            if(dphigh .lt. dplow) index = index + 1
            do 425 kk = index, nperiods
               use_it(kk) = -1
425         continue
          end if
       end if

       if(key .eq. 'p' .or. key .eq. 'P') then
          if(idevice .eq. iPS .or. idevice .eq. iCPS)then
            call pgiden
	    call pgclos
	    call pgslct(itermstat)
	    idevice = iold_device
            ideviceChanged = 0
            key = 'r'
          else
            iold_device = idevice
	    write(stdout,*)'Plot file name must end with /ps or /cps'
	    idevice = iCPS
            ideviceChanged = 1
            key = 'r'
          end if
       end if
          
       if(key .eq. 'w' .or. key .eq. 'W')then
         call put_dsprsn(seisfile,splGroupVels,
     &           periods,theInstPeriod,
     &           c_amps,use_it,nperiods,itype,
     &           theGroupVels, gvlow, gvhigh,
     &           gwidth,variable_gwidth,
     &           isolated,pmin,pmax,vgmin,vgmax,errfraction,
     &           is_overtone)
       end if
*
*      variable Gaussian width 
*
       if(key .eq. 'g' .or. key .eq. 'G')then
         write(stdout,'(a,$)') 'Enter 1 for variable gwidth: '
450      read(stdin,'(i1)',err=450) variable_gwidth 
         if(variable_gwidth .ne. 1)then
	    write(stdout,'(a)')'Enter new value of gwidth: '
	    write(stdout,451) gwidth
451         format(' (current value = ',f6.2,' )',$)
452         read(stdin,*,err=452) gwidth
	 end if
*        force a complete update
         isolated = 1
	 key = 'u'
       end if
*
       if(key .eq. 'u' .or. key .eq. 'U')then
         do 505 k=1,nperiods
          use_it(k) = 1
505      continue
         do 508 i = 1, nfpts
           seis(i) = orig_seis(i)
508      continue      
*        force an update
         if(isolated .eq. 1) then
	    isolated = 0
	    go to 250
	 else
	    key = 'r'
	 end if
       end if
*
*      perform a phase match and plot the results
*     
       if(key .eq. 'c' .or. key .eq. 'C') then
          call sphs_match(wind_seis,nfpts,nfpts,dt,periods,
     &      splGroupVels,nperiods,distance,or_beg,0)
*
          isolated = 1
*
          do 550 i = 1, nfpts
             seis(i) = wind_seis(i)
550       continue
*         force a complete update
          go to 250
       end if
*
*      perform a phase match and plot the results
*     
       if(key .eq. 'b' .or. key .eq. 'B') then
          call sphs_match(wind_seis,nfpts,nfpts,dt,periods,
     &      splGroupVels,nperiods,distance,or_beg,1)
*
          isolated = 1
*
          do 555 i = 1, nfpts
             seis(i) = wind_seis(i)
555       continue
*         force a complete update
          go to 250
       end if
*
*      perform a phase match with reference model and plot the results
*     
       if((key .eq. 'x') .and. ihasrefcurve .eq. 1) then
          call sphs_match(wind_seis,nfpts,nfpts,dt,
     &      prem_t,prem_g,nperiods,distance,or_beg,0)
*
          isolated = 1
*
          do 554 i = 1, nfpts
             seis(i) = wind_seis(i)
554       continue
*         force a complete update
          go to 250
       end if
*
*      perform a phase match with reference model and plot the results
*     
       if((key .eq. 'y') .and. ihasrefcurve .eq. 1) then
          call sphs_match(wind_seis,nfpts,nfpts,dt,
     &      prem_t,prem_g,nperiods,distance,or_beg,1)
*
          isolated = 1
*
          do 556 i = 1, nfpts
             seis(i) = wind_seis(i)
556       continue
*         force a complete update
          go to 250
       end if
*
*
*      perform a phase match and keep the residual (remove the mode)
*     
       if(key .eq. 'o' .or. key .eq. 'O') then
          call sphs_match(wind_seis,nfpts,nfpts,dt,periods,
     &      splGroupVels,nperiods,distance,or_beg,0)
*
          isolated = 1
	  is_overtone = 1
*
          do 557 i = 1, nfpts
             seis(i) = seis(i) - wind_seis(i)
557       continue
*         force a complete update
          go to 250
       end if

       if(key .eq. 'k' .or. key .eq. 'K')then
         write(stdout,'(a)') 'How many knots in the spline fit?'
	 write(stdout,558) n_splines
558      format(' (current value is ',i2.2,') ',$)
560      read(stdin,*,err=560) n_splines
*        force an update
         key = 'r'
       end if
*
       if(key .eq. 's' .or. key .eq. 'S')then
         call put_seis(seis,ntpts,dt,normalizer)
       end if
*
*
*      view the values in the terminal window
*       
       if(key .eq. 'v' .or. key .eq. 'V')then
         write(stdout,*)' '
	 write(stdout,564)
564      format(66('-'))
	 write(stdout,565)
565      format('Fltr_Period',2x,'Inst_Period',2x,
     &   'Spline_Vel',2x,'Peak_Vel',1x,'Centroid_Vel',2x,'%vdif')
	 write(stdout,564)
         do 1000 i = 1, nperiods
           if(use_it(i) .gt. 0)then
           write(stdout,566) 10**periods(i),theInstPeriod(i),
     &       splGroupVels(i), theGroupVels(i),centroidGroupVels(i),
     &       100*(theGroupVels(i) - centroidGroupVels(i))/theGroupVels(i)
566        format(2x,f7.2,5x,f7.2,5x,3(f7.3,4x),2x,f4.1)
           end if
 1000    continue
	 write(stdout,564)
*       
       end if
*
*      adjust the color range 0.5 is default 
*
       if(key .eq. 'a')then
         write(stdout,'(a)') 'Enter new brightness and contrast values'
	 write(stdout,568)bright, contrast
568      format(' current: ',f5.2,1x,f5.2,' (defaults: 0.5, 0.5) ',$)
570      read(stdin,*,err=570) bright, contrast 
	 key = 'r'
       end if
*
       if(key .eq. 'z' .or. key .eq. 'Z')then
         write(stdout,'(a)') 'What is the exponent for image scaling?'
	 write(stdout,578) a_image_exponent
578      format(' (current value is ',f5.3,') ',$)
580      read(stdin,*,err=580) a_image_exponent
*        force a complete update
         go to 250
       end if
*
*      output some signals for examination 
*
       if(key .eq. 'e' .or. key .eq. 'E')then
         write(stdout,'(a)') 'Enter the filter center period to examine'
590      read(stdin,*,err=590) examine_period
         i_examine_period = 1
*        force a complete update
         go to 250
      end if
*

       if(key .eq. 'n' .or. key .eq. 'N') goto 8888
*
       if(key .eq. 'r' .or. key .eq. 'R' .or. key .eq. 'X') goto 599
*
*********************************************************************************      
*   loop back
*********************************************************************************       
       go to 490
*
*********************************************************************************      
*    get out
*********************************************************************************
599     continue
*
********************************************************************************
       if(key .eq. 'r'.or. key .eq. 'R' .or. key .eq. 'X') go to 300

*********************************************************************************
*     Write the image to a file
*
*********************************************************************************
*       open(unit=ounit, file='image')
*       write(ounit,*) nperiods, nvel
*       do 601 i = 1, nvel
*          do 600 j = 1, nperiods
*             k = (j-1)*nvel + (nvel - (i-1))
*             write(ounit,*) iobuffer(k)
*600       continue
*601    continue
*       close(ounit)
*
        end if
*       end if the if construct associated with a batch run
*
8888  continue
*      write(stdout,*)'Going back for the next file.'
      go to 10
      
9999  continue
      stop
      end
      
****************************************************************
*
*    calculate extrema
*
****************************************************************

      subroutine getminmeanmax(x,npts,min,mean,max)
      real x(*), min,max,mean
      integer npts

      real sum
      integer i
      
      min =  1e20
      max = -1e20
      sum = 0.0
      do 1 i = 1, npts
       if(x(i) .gt. max) max = x(i)
       if(x(i) .lt. min) min = x(i)
       sum = sum + x(i)
1     continue
      mean = sum / float(npts)

      return
      end

***********************************************************************
*
*    Get SubImage Max & Min
*
***********************************************************************
      subroutine get_subimage_range(x,mr,mc,nvel,nperiods,use_it,cmin,cmax)
      integer use_it(nperiods),nvel,nperiods
      real x(mr,mc), cmin,cmax
       
      cmin =  1e20
      cmax = -1e20
      do 2 i = 1, nperiods
          if(use_it(i) .eq. 1)then
             do 1 k = 1, nvel
	        tmp = x(i,k)
		if(tmp .lt. cmin) cmin = tmp
		if(tmp .gt. cmax) cmax = tmp
1            continue
          end if
2     continue

      return
      end
***********************************************************************
*
*    Get bounds on velocity for a particular period
*      returns the indices in the array v! you have to
*      convert those to velocity.
*    
*    v = envelope time series
*    np = npts in v
*    imax = index of max value in v
*    ih = index of high value
*    il = index of low value
*    fraction = value of peak to use for limits
*
***********************************************************************
      subroutine bound_it(v,np,imax,ih,il,fraction)
      real v(*)
      integer np, imax, ih, il
*     
*     find the high index
*
      pv = v(imax)
      thevalue = fraction * pv
*
      ih = imax
1     continue
      ih = ih - 1
      if(v(ih) .gt. thevalue .and. ih .ge.  2) go to 1
*     
*     find the low index
*
      il = imax
2     continue
      il = il + 1
      if(v(il) .gt. thevalue .and. il .le. np-1) go to 2
*     
      continue
*    
      return     
      end

       SUBROUTINE PALETT(TYPE, CONTRA, BRIGHT)
C-----------------------------------------------------------------------
C Set a "palette" of colors in the range of color indices used by
C PGIMAG.
C-----------------------------------------------------------------------
      INTEGER TYPE
      REAL CONTRA, BRIGHT
C
      REAL GL(2), GR(2), GG(2), GB(2)
      REAL RL(9), RR(9), RG(9), RB(9)
      REAL HL(5), HR(5), HG(5), HB(5)
      REAL WL(10), WR(10), WG(10), WB(10)
      REAL AL(20), AR(20), AG(20), AB(20)
C
      DATA GL /0.0, 1.0/
      DATA GR /0.0, 1.0/
      DATA GG /0.0, 1.0/
      DATA GB /0.0, 1.0/
C
      DATA RL /-0.5, 0.0, 0.17, 0.33, 0.50, 0.67, 0.83, 1.0, 1.7/
      DATA RR / 0.0, 0.0,  0.0,  0.0,  0.6,  1.0,  1.0, 1.0, 1.0/
      DATA RG / 0.0, 0.0,  0.0,  1.0,  1.0,  1.0,  0.6, 0.0, 1.0/
      DATA RB / 0.0, 0.3,  0.8,  1.0,  0.3,  0.0,  0.0, 0.0, 1.0/
C
      DATA HL /0.0, 0.2, 0.4, 0.6, 1.0/
      DATA HR /0.0, 0.5, 1.0, 1.0, 1.0/
      DATA HG /0.0, 0.0, 0.5, 1.0, 1.0/
      DATA HB /0.0, 0.0, 0.0, 0.3, 1.0/
C
      DATA WL /0.0, 0.5, 0.5, 0.7, 0.7, 0.85, 0.85, 0.95, 0.95, 1.0/
      DATA WR /0.0, 1.0, 0.0, 0.0, 0.3,  0.8,  0.3,  1.0,  1.0, 1.0/
      DATA WG /0.0, 0.5, 0.4, 1.0, 0.0,  0.0,  0.2,  0.7,  1.0, 1.0/
      DATA WB /0.0, 0.0, 0.0, 0.0, 0.4,  1.0,  0.0,  0.0, 0.95, 1.0/
C
      DATA AL /0.0, 0.1, 0.1, 0.2, 0.2, 0.3, 0.3, 0.4, 0.4, 0.5,
     :         0.5, 0.6, 0.6, 0.7, 0.7, 0.8, 0.8, 0.9, 0.9, 1.0/
      DATA AR /0.0, 0.0, 0.3, 0.3, 0.5, 0.5, 0.0, 0.0, 0.0, 0.0,
     :         0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0/
      DATA AG /0.0, 0.0, 0.3, 0.3, 0.0, 0.0, 0.0, 0.0, 0.8, 0.8,
     :         0.6, 0.6, 1.0, 1.0, 1.0, 1.0, 0.8, 0.8, 0.0, 0.0/
      DATA AB /0.0, 0.0, 0.3, 0.3, 0.7, 0.7, 0.7, 0.7, 0.9, 0.9,
     :         0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0/
C
      IF (TYPE.EQ.1) THEN
C        -- gray scale
         CALL PGCTAB(GL, GR, GG, GB, 2, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.2) THEN
C        -- rainbow
         CALL PGCTAB(RL, RR, RG, RB, 9, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.3) THEN
C        -- heat
         CALL PGCTAB(HL, HR, HG, HB, 5, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.4) THEN
C        -- weird IRAF
         CALL PGCTAB(WL, WR, WG, WB, 10, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.5) THEN
C        -- AIPS
         CALL PGCTAB(AL, AR, AG, AB, 20, CONTRA, BRIGHT)
      END IF
      END
           
c      
      subroutine fillrect(icolor,wmin,wmax,vgmin,vgmax)
c
       real boxx(5), boxy(5)
       call pgsave
       call pgsci(icolor)
       boxx(1) = wmin
       boxy(1) = vgmin
       boxx(2) = wmin
       boxy(2) = vgmax
       boxx(3) = wmax
       boxy(3) = vgmax
       boxx(4) = wmax
       boxy(4) = vgmin
       boxx(5) = boxx(1)
       boxy(5) = boxy(1)
       call pgpoly(5,boxx,boxy)
       call pgunsa
       
       return
       end
*
************************************************************************
************************************************************************
************************************************************************
************************************************************************
*
       subroutine chkopts(nargc,batch,variable_gwidth,estimate_noise)
       integer nargc,icmd,variable_gwidth,estimate_noise
       logical batch
       character*70 cmdlin

       batch = .false.
*
*      loop through the arguments
       do 10  icmd = 1,nargc
	  call getarg( icmd, cmdlin )
	  if ( cmdlin(1:2) .eq. "-b" ) then
	     batch = .true.
	  else if ( cmdlin(1:2) .eq. "-f" ) then
	            variable_gwidth = 0
	  else if ( cmdlin(1:2) .eq. "-n" ) then
	            estimate_noise = 1
	  else
	     write(*,*) 'Illegal Command Line Argument',cmdlin
	     write(*,*) 'Valid Command Line Arguments:'
	     write(*,*) '-b (Batch mode processing, non-interactive)'
	     write(*,*) 'Default is fully interactive processing'
	     write(*,*) '-f (fixed gaussian width parameter)'
	     write(*,*) 'Default is variable gaussian width parameter'
	  endif
 10    continue
       return
       end
*
************************************************************************
************************************************************************
************************************************************************
************************************************************************
*
       subroutine listmenu()
      
       integer stdout
       
       stdout = 6
       
       write(stdout,*) ' '
       write(stdout,'(a)')'**********************************************'
       write(stdout,*)'MENU: '
       write(stdout,*)'m = list this menu'
       write(stdout,*)'r = refresh the plot'
       write(stdout,*)'l = truncate all periods lower'
       write(stdout,*)'h = truncate all periods higher'
       write(stdout,*)'u = use all periods (undoes l or h command)'
       write(stdout,*)'d = remove value at cursor position'
       write(stdout,*)'i = set velocity to cursor location'
       write(stdout,*)'w = write the dispersion'
       write(stdout,*)'n = go on to next waveform'
       write(stdout,*)'k = change number of knots in spline fit'
       write(stdout,*)'v = list the values to the terminal'
       write(stdout,*)'p = save postscript file'
       write(stdout,*)'c = clean with mode-isolation filter'
       write(stdout,*)'b = batch clean with mode-isolation filter'
       write(stdout,*)'x = clean with mode-isolation filter (reference curve)'
       write(stdout,*)'y = batch clean with mode-isolation filter (reference curve)'
       write(stdout,*)'g = change Gaussian width parameter'
       write(stdout,*)'s = save the seismogram to a file'
       write(stdout,*)'a = adjust the color range'
       write(stdout,*)'z = adjust the image scaling factor'
       write(stdout,*)'e = examine signals for a specific period'
       write(stdout,*)'q = quit'
       write(stdout,'(a)')'**********************************************'
       write(stdout,*)'command-line arguments:'
       write(stdout,*)'-b invokes batch operation'
       write(stdout,*)'-f defaults to fixed Gaussian width parameter'
       write(stdout,*)'   which is read in from in.swmfa'
       write(stdout,*)'-n use pre-signal noise in plot of spectrum'
       write(stdout,'(a)')'**********************************************'
       
       return
       end
      
      
