****************************************************************
*
*    Write out the SAC file of group velocities
*
*    v contains the splined velocities
*    vp contains the peak velocities
*    p contains the log10 of the period
*    amps contains the complex sepctral value
*    use_it contains a flag for use it
*    n is the number of periods
*    r_or_l identifies rayleigh of love (for name)
*    gl, gh  are error estimates for the peak width
*
****************************************************************
      subroutine put_dsprsn(fn,v,p,ip,amps,use_it,n,
     &                      r_or_l,vp,gl,gh,
     &                      alpha,variable_alpha,
     &                      isolated,pmin,pmax,
     &                      vgmin,vgmax,efrac,
     &                      is_overtone)
      character*256 fn
      real v(n),vp(n), p(n), ip(n), gl(n),gh(n)
      complex amps(n)
      integer n, use_it(n),r_or_l,variable_alpha
      integer is_overtone
      
      real x(512),y(512)
      integer i, j, jdt(6)
      character*32 kstring
      character*8 kstnm, kcmpnm
      character*256 suffix,outfile,outfile2
      character*3 km
      character*64 user_name, todays_date, aformat
      character*256 thedir
      
      ichannelname = 1

      call getenv( 'USER', user_name )
      call getenv( 'PWD', thedir )
      call fdate(todays_date)
c
c     fill the output names with blanks
c
      do 11 i = 1, 256
	  outfile(i:i) = " "
	 outfile2(i:i) = " "
11    continue
c
      iounit = 12 
      
      j = 0
      
      do 1 i = 1, n
         if(use_it(i) .eq. 1)then
           j = j + 1
           x(j) = ip(i)
           y(j) = v(i)
          end if
1     continue

      if(j .gt. 0) then
*
*        build the name
*
         call getnhv('NZYEAR',jdt(1),nerr)
         call getnhv('NZJDAY',jdt(2),nerr)
         call getnhv('NZHOUR',jdt(3),nerr)
         call getnhv('NZMIN' ,jdt(4),nerr)
         call getnhv('NZSEC' ,jdt(5),nerr)
         jdt(6) = 0
         call getfhv('O',t0,nerr)
         jdt(5) = jdt(5) + int(tstart)
*
	 call getdate(jdt(1),jdt(2),im,id)
*      
         call getkhv('kstnm',kstnm,nerr)
	 call getkhv('KCMPNM',kcmpnm,nerr)
*	 
         ilen  = lnblnk(kstnm)
         ilen2 = lnblnk(kcmpnm)
         suffix(1:1) = '_'
         if(ichannelname .eq. 0)then
             if(r_or_l .eq. 1)then
               suffix(1:ilen+5)=suffix(1:1)//kstnm(1:ilen)//'_r.g'
             else if(r_or_l .eq. 2)then
               suffix(1:ilen+5)=suffix(1:1)//kstnm(1:ilen)//'_l.g'
             end if
	     ilensuffix = ilen + 5
	 else
             suffix(1:ilen+2) = suffix(1:1)//kstnm(1:ilen)//'_'
             if(r_or_l .eq. 1)then
               suffix(1:ilen+ilen2+6) = 
     &          suffix(1:ilen+2)//kcmpnm(1:ilen2)//'_r.g'
             else if(r_or_l .eq. 2)then
               suffix(1:ilen+ilen2+6) = 
     &          suffix(1:ilen+2)//kcmpnm(1:ilen2)//'_l.g'
             end if
	     ilensuffix = ilen + ilen2 + 6
         end if
*
* 
	 iy = jdt(1) - 1900
	 if(jdt(1) .ge. 2000) iy = jdt(1) - 2000
*	 
         write(outfile,3) iy,im,id,jdt(3),jdt(4)
3        format(i2.2,'_',i2.2,'_',i2.2,'_',i2.2,'_',i2.2)


         i1 = lnblnk(outfile)
        i2 = ilensuffix
         outfile(1:i1+i2) = outfile(1:i1)//suffix(1:i2)
	 if(is_overtone .eq. 1)then
           outfile2(1:i1+i2+1) = outfile(1:i1)//suffix(1:i2)//'o'
	 else
           outfile2(1:i1+i2+1) = outfile(1:i1)//suffix(1:i2)//'a'
	 end if
*
*        write out the SAC file (don't permanently change header)
*         
*        store the header values that will be changed	 
*
*        call getihv('IFTYPE',itype,nerr)
         call getfhv('B',tmp_b,nerr)
         call getfhv('E',tmp_e,nerr)
         call getnhv('NPTS',inpts,nerr)
	 call getfhv('O',tmp_o,nerr)
*
*        change those header values
*	 
         call setihv('IFTYPE','IXY',nerr)
         call setlhv('LEVEN',.false.,nerr)
         call setfhv('B',x(1),nerr)
         call setfhv('E',x(j),nerr)
         call setnhv('NPTS',j,nerr)
	 call setfhv('O',-12345.,nerr)
*
*        write the SAC file
*         
 	 call wsac0(outfile,x,y,nerr)
*
*        restore the changed header values	 
*
         call setihv('IFTYPE','ITIME',nerr)
         call setlhv('LEVEN',.true.,nerr)
         call setnhv('NPTS',inpts,nerr)
	 call setfhv('O',tmp_o,nerr)
	 call setfhv('B',tmp_b,nerr)
	 call setfhv('E',tmp_e,nerr)
*
*        write an ASCII file
*
*        Look up some info 
*	 
	 call getfhv('STLA',stla,nerr)
	 call getfhv('STLO',stlo,nerr)
	 call getfhv('EVLA',evla,nerr)
	 call getfhv('EVLO',evlo,nerr)
	 call getfhv('EVDP',evdp,nerr)
	 	 
	 call getkhv('KCMPNM',kcmpnm,nerr)
*
         open(unit = iounit, file=outfile2)
*
         write(iounit,'(a21)') '# SWMFA Version 3.9.0'
         write(iounit,100)kstnm,stla,stlo,kcmpnm
100      format('# Station Info:',1x,a6,2x,f8.4,2x,f10.4,2x,a10)
*
         write(iounit,200)evla,evlo,evdp
200      format('# Event Info:',2x,f8.4,2x,f10.4,2x,g7.2)
*
         write(iounit,300) user_name, todays_date
300      format('# User and Time Processed: ',a10,3x,a16)
*
         i1 = lnblnk(fn)
         i1 = fslen(fn)
	 write(aformat,'(a6,i2.2,a1)')'(a13,a',i1,')'
         write(iounit,aformat) '# File name: ',fn
         write(iounit,'(a12)')'#  Directory:'
         write(iounit,400)  thedir
400      format('#   ',a70)
*
	 if(variable_alpha .eq. 1)then
	    write(iounit,500) -99.0, isolated, efrac
	 else
            write(iounit,501) alpha, isolated, efrac
         end if
500      format('# Filter Width Parameter: ','V',1x,f6.2,2x,
     &    'Mode Isolation: ',i1.1,2x,'Peak Error Fraction: ',f4.2)
501      format('# Filter Width Parameter: ','C',1x,f6.2,2x,
     &    'Mode Isolation: ',i1.1,2x,'Peak Error Fraction: ',f4.2)
*	 
	 write(iounit,600) 10.0**pmin, 10.0**pmax
600      format('# Period Range Processed: ',f7.2,2x,f7.2)
*
         write(iounit,700) vgmin, vgmax
700      format('# Group Velocity Range Processed: ',f7.2,2x,f7.2)
*
	 if(is_overtone .eq. 1)then
	   write(iounit,800)
         end if
800      format('# Overtone')
         write(iounit,900)
900      format('#',/'#',3x,'T0',4x,'InstT',4x,'spV',5x,'pkV', 
     &    5x,'ldV',5x,'hdV',8x,'Real(A)',9x,'Imag(A)')	  
           do 10 k=1,n
            if(use_it(k) .gt. 0)then
               write(iounit,12) 10**p(k),ip(k),v(k),vp(k),
     &                           vp(k)-gl(k),gh(k)-vp(k),amps(k)
            end if
10         continue
           close(iounit)
12         format(2(f7.2,1x),4(f7.3,1x),1x,2(e15.6,1x))
*          
           write(stdout,'(a17,a40)') ' Just wrote out: ',outfile
*
       end if      
       return
       end
       
****************************************************************
*
*    Convert the month string to an integer
*
****************************************************************
       subroutine getmonth(kmonth,imonth)
       character*3 kmonth
       integer imonth
       
       character*4 theMonths(12)
       data theMonths/'JAN ','FEB ','MAR ','APR ','MAY ','JUN ',
     &            'JUL ','AUG ','SEP ','OCT ','NOV ','DEC '/
     
        do 1 i = 1,12
          if(kmonth .eq. theMonths(i)) imonth = i
1       continue

        return
        end
****************************************************************
*
*    Convert the day of year to a month and date
*
****************************************************************
       subroutine getdate(iyear,idoy,imonth,iday)
       integer idoy,imonth,iday,lp,ndays(12),remainder
       data ndays/31,28,31,30,31,30,31,31,30,31,30,31/  
* 
       lp = 0
       i0 = remainder(iyear,4)
       i1 = remainder(iyear,100)
       i2 = remainder(iyear,400)
       if((i0 .eq. 0 .and. i1 .ne. 0) .or. i2 .eq. 0) lp = 1
*      
       if(lp .eq. 1)then
         ndays(2)=29
       endif
  
       iday = idoy
       do 1  i = 1,12
        if(iday .le. ndays(i)) go to 999
        iday = iday - ndays(i)
1      continue
*
999    continue
       imonth = i 
*
9999   return
       end
****************************************************************
*
*    compute the reaminder of an integer division
*
****************************************************************
	
	integer function remainder(numerator,denominator)
	integer numerator,denominator
	
	i1 = numerator/denominator
	i2 = i1 * denominator
	
	remainder = numerator - i2
	
	return
	end
	
	
