c
c    fit a set of points with a quadratic equation
c       and return the value of the maximum
c
c     y =  m(1) * x^2 + m(2) * x + m(3)
c
c     extrema will occur when 2 * m(1) * x + m(2) = 0
c       or x = -m(2) / (2 * m(1))
c
      subroutine quad_peak(x,y,n,xpeak)
      real x(n), y(n), xpeak
      integer n
      real g(3,3),d(3)
c
c     clean out the matrices
c      
      do 1 i = 1, 3
         d(i) = 0.0
         do 1 j = 1,3
	    g(i,j) = 0.0
1     continue
c      
c     compute the system of equations
c
      do 10 i = 1, n
	g(1,1) = g(1,1) + x(i)*x(i)*x(i)*x(i)
	g(1,2) = g(1,2) + x(i)*x(i)*x(i)
	g(1,3) = g(1,3) + x(i)*x(i)	
	g(2,3) = g(2,3) + x(i)
c	
	d(1) = d(1) + x(i)*x(i)*y(i)	
	d(2) = d(2) + x(i)*y(i)	
	d(3) = d(3) + y(i)	
10    continue
        g(2,1) = g(1,2)
        g(2,2) = g(1,3)
        g(3,1) = g(1,3)
        g(3,2) = g(2,3)
        g(3,3) = real(n)
c
c       solve the algebra
c
        call gaussj(g,3,3,d,1,1)
c
        xpeak = -d(2) / (2 * d(1))
c	
	return
	end
c	
c		
      SUBROUTINE GAUSSJ(A,N,NP,B,M,MP)
      PARAMETER (NMAX=6)
      DIMENSION A(NP,NP),B(NP,MP),IPIV(NMAX),INDXR(NMAX),INDXC(NMAX)
      DO 11 J=1,N
        IPIV(J)=0
11    CONTINUE
      DO 22 I=1,N
        BIG=0.
        DO 13 J=1,N
          IF(IPIV(J).NE.1)THEN
            DO 12 K=1,N
              IF (IPIV(K).EQ.0) THEN
                IF (ABS(A(J,K)).GE.BIG)THEN
                  BIG=ABS(A(J,K))
                  IROW=J
                  ICOL=K
                ENDIF
              ELSE IF (IPIV(K).GT.1) THEN
                PAUSE 'Singular matrix'
              ENDIF
12          CONTINUE
          ENDIF
13      CONTINUE
        IPIV(ICOL)=IPIV(ICOL)+1
        IF (IROW.NE.ICOL) THEN
          DO 14 L=1,N
            DUM=A(IROW,L)
            A(IROW,L)=A(ICOL,L)
            A(ICOL,L)=DUM
14        CONTINUE
          DO 15 L=1,M
            DUM=B(IROW,L)
            B(IROW,L)=B(ICOL,L)
            B(ICOL,L)=DUM
15        CONTINUE
        ENDIF
        INDXR(I)=IROW
        INDXC(I)=ICOL
        IF (A(ICOL,ICOL).EQ.0.) PAUSE 'Singular matrix.'
        PIVINV=1./A(ICOL,ICOL)
        A(ICOL,ICOL)=1.
        DO 16 L=1,N
          A(ICOL,L)=A(ICOL,L)*PIVINV
16      CONTINUE
        DO 17 L=1,M
          B(ICOL,L)=B(ICOL,L)*PIVINV
17      CONTINUE
        DO 21 LL=1,N
          IF(LL.NE.ICOL)THEN
            DUM=A(LL,ICOL)
            A(LL,ICOL)=0.
            DO 18 L=1,N
              A(LL,L)=A(LL,L)-A(ICOL,L)*DUM
18          CONTINUE
            DO 19 L=1,M
              B(LL,L)=B(LL,L)-B(ICOL,L)*DUM
19          CONTINUE
          ENDIF
21      CONTINUE
22    CONTINUE
      DO 24 L=N,1,-1
        IF(INDXR(L).NE.INDXC(L))THEN
          DO 23 K=1,N
            DUM=A(K,INDXR(L))
            A(K,INDXR(L))=A(K,INDXC(L))
            A(K,INDXC(L))=DUM
23        CONTINUE
        ENDIF
24    CONTINUE
      RETURN
      END
